﻿#include "warning.h"
#include "ui_warning.h"

#include <QPainter>
#include <QLayout>
#include <QDebug>
#include <QFile>

#include "oneshadow.h"

Warning::Warning(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::Warning)
{
    ui->setupUi(this);

    /* 加载qss */
    QFile fileQss(":/qss/warning/warning_dark.qss");
    if(fileQss.open(QFile::ReadOnly))
    {
        QString qss = fileQss.readAll();
        this->setStyleSheet(qss);
        fileQss.close();
    } else
    {
        qDebug() << "加载QSS文件失败";
    }

    /* 设置无边框和背景透明 */
    this->setWindowFlags(Qt::Dialog | Qt::FramelessWindowHint);
    this->setAttribute(Qt::WA_TranslucentBackground);

    /* 设置文字自动换行 */
    ui->label_Warn->setWordWrap(true);

    /* 阴影宽度是16 */
    this->layout()->setMargin(SHADOW_W);
    m_shadow = new OneShadow(QSize(width() - SHADOW_W*2, height() - SHADOW_W*2),SHADOW_W);

    connect(ui->pBtn_close,SIGNAL(clicked()),this,SLOT(close()));
    connect(ui->pBtn_cancel,SIGNAL(clicked()),this,SLOT(close()));
    connect(ui->pBtn_ok,SIGNAL(clicked()),this,SLOT(do_ok()));
}

Warning::~Warning()
{
    delete ui;
}

void Warning::setText(const QString &text)
{
    /* 根据文本大小设置高度 */
    
    /* 设置文本居中 */
    ui->label_Warn->setAlignment(Qt::AlignCenter);
    ui->label_Warn->setText(text);
}
/* 只有一个确定按钮 */
void Warning::setTextWithOneButton(const QString &text)
{
    /* 设置文本居中 */
    ui->label_Warn->setAlignment(Qt::AlignCenter);
    ui->pBtn_cancel->hide();
    // ui->pBtn_ok->move(ui->pBtn_cancel->x(),ui->pBtn_cancel->y());
    ui->label_Warn->setText(text);
}

void Warning::paintEvent(QPaintEvent *event)
{
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);
    painter.drawImage(QPoint(0,0),m_shadow->image());
}



void Warning::do_ok()
{
    emit signal_ok();
    m_isOk = true;
    this->close();
}
